{-

The main application program

Command line arguments: infilePath, outfilePath, itrs, [mode]

infilePath:  path of input file, which should be in the format of lines 
             consisting of `fromNode toNode`
outfilePath: path of output file
itrs:        number of iterations in the PageRank computation
mode:        optional, mode of PageRank computation, one of {seq, mr_def,
             mr_ext}, default to mr_def 
             seq:    non-parallel sequential computation
             mr_def: parallel implementation based on default MapReduce
             mr_ext: benchmark parallel implementation based on external opensourced MapReduce library

-}

module Main (main) where

import Control.Monad (when)
import System.IO (openFile, IOMode(WriteMode), hPutStrLn, hClose)
import System.Environment (getArgs, getProgName)
import System.Exit
import Data.Map as M (toList)

import ProcessData (processData)
import PageRank (computePageRankSeq, computePageRankMR)
import PageRankExt (computePageRankMRext)

main :: IO()
main = do
    progName <- getProgName
    args <- getArgs

    when (length args /= 3 && length args /= 4) $
        die $ "Usage: " ++ progName ++ " <infilePath> <outfilePath> <itrs> [mode], where\
        \ mode is one of {seq, mr_def, mr_ext}, default to mr_def"

    let infilePath : outfilePath : itrs : mode = args
        computePageRank = case mode of
            [] -> computePageRankMR
            ["mr_def"] -> computePageRankMR
            ["seq"] -> computePageRankSeq
            ["mr_ext"] -> computePageRankMRext
            _ -> error $ "Usage: " ++ progName ++ " <infilePath> <outfilePath> <itrs> [mode], where\
            \ mode is one of {seq, mr_def, mr_ext}, default to mr_def"

    (graph, pageRank) <- processData infilePath
    let resPageRank = computePageRank pageRank graph (read itrs) 0.85
    h <- openFile outfilePath WriteMode
    mapM_ (hPutStrLn h) [ n ++ ": " ++ show pr | (n, pr) <- M.toList resPageRank ]
    hClose h
